﻿using Microsoft.Extensions.Logging;
using ShoeStore.Repositories;

namespace ShoeStore.Reports;

internal class DocReport
{
    private readonly IStoreRepository _storeRepository;//IStoreRepository _storeRepository
    private readonly IFactoryRepository _factoryRepository;
    private readonly IShoesRepository _shoesRepository;

    private readonly ILogger<DocReport> _logger;

    public DocReport(IStoreRepository storeRepository, IFactoryRepository factoryRepository,
    IShoesRepository shoesRepository, ILogger<DocReport> logger)
    {
        _storeRepository = storeRepository ?? throw new ArgumentNullException(nameof(storeRepository));
        _factoryRepository = factoryRepository ?? throw new ArgumentNullException(nameof(factoryRepository));
        _shoesRepository = shoesRepository ?? throw new ArgumentNullException(nameof(shoesRepository));
        _logger = logger ?? throw new ArgumentNullException(nameof(logger));
    }

    public bool CreateDoc(string filePath, bool includeStore, bool includeFactorys, bool includeShoess)
    {
        try
        {
            var builder = new WordBuilder(filePath).AddHeader("Документ со справочниками");

            if (includeStore)
            {
                builder.AddParagraph("Магазины").AddTable([2400, 2400, 1200], GetStores());
            }

            if (includeFactorys)
            {
                builder.AddParagraph("Фабрики").AddTable([2400, 2400 ], GetFactorys());
            }

            if (includeShoess)
            {
                builder.AddParagraph("Обувь").AddTable([2400, 2400], GetShoess());
            }

            builder.Build();

            return true;
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при формировании документа");
            return false;
        }
    }

    private List<string[]> GetStores()
    {
        return [
        ["Название", "Работники", "Посетители"],
        .. _storeRepository
            .ReadStores()
            .Select(x => new string[] { /*x.AnimalSpecies,*/ x.StoreName, x.Employees.ToString(), x.Visitors.ToString() }),
        ];
    }

    private List<string[]> GetFactorys()
    {
        return [
        ["Название", "Страна"],
        .. _factoryRepository
            .ReadFactorys()
            .Select(x => new string[] { x.FactoryName/*, x.LastName*/, x.Manufacturer.ToString() }),
        ];
    }

    private List<string[]> GetShoess()
    {
        return [
        ["Название", "Описание"],
        .. _shoesRepository
            .ReadShoess()
            .Select(x => new string[] { /*x.ShoesType.ToString(),*/ x.Name, x.Description }),
        ];
    }
}
